@@ -39,6 +39,9 @@ static size_t gzwrite(gzFile file, const void *buff, size_t size) {
     return fwrite(buff, size, 1, file);
 }
 static int gzeof(gzFile file) {return feof(file);}
+static int gzseek(gzFile file, long offset, int whence) {
+    return fseek(file, offset, whence);
+}
 
 #endif
 
@@ -65,6 +68,11 @@ static int gzeof(gzFile file) {return feof(file);}
  *
  *  chunks types:
  *
+ *  IMG : a dict of info:
+ *      - box: the image gox.
+ *
+ *  PREV: a png image for preview.
+ *
  *  BL16: a 16^3 block saved as a 64x64 png image.
  *
  *  LAYR: a layer:
@@ -131,7 +139,10 @@ static void chunk_read(chunk_t *c, gzFile in, char *buff, int size)
 {
     c->pos += size;
     assert(c->pos <= c->length);
-    gzread(in, buff, size);
+    if (buff)
+        gzread(in, buff, size);
+    else
+        gzseek(in, size, SEEK_CUR);
 }
 
 static int32_t chunk_read_int32(chunk_t *c, gzFile in)
@@ -213,7 +224,7 @@ static void chunk_write_all(gzFile out, const char *type,
     write_int32(out, 0);        // CRC XXX: todo.
 }
 
-void save_to_file(goxel_t *goxel, const char *path)
+void save_to_file(goxel_t *goxel, const char *path, bool with_preview)
 {
     // XXX: remove all empty blocks before saving.
     LOG_I("Save to %s", path);
@@ -223,11 +234,15 @@ void save_to_file(goxel_t *goxel, const char *path)
     int nb_blocks, index, size, bpos[3];
     uint64_t uid;
     gzFile out;
-    uint8_t *png;
+    uint8_t *png, *preview;
     camera_t *camera;
     mesh_iterator_t iter;
 
     out = gzopen(path, str_endswith(path, ".gz") ? "wb" : "wbT");
+    if (!out) {
+        LOG_E("Cannot save to %s", path);
+        return;
+    }
     gzwrite(out, "GOX ", 4);
     write_int32(out, VERSION);
 
@@ -238,6 +253,15 @@ void save_to_file(goxel_t *goxel, const char *path)
                                sizeof(goxel->image->box));
     chunk_write_finish(&c, out);
 
+    if (with_preview) {
+        preview = calloc(128 * 128, 4);
+        goxel_render_to_buf(preview, 128, 128, 4);
+        png = img_write_to_mem(preview, 128, 128, 4, &size);
+        chunk_write_all(out, "PREV", (char*)png, size);
+        free(preview);
+        free(png);
+    }
+
     // Add all the blocks data into the hash table.
     index = 0;
     DL_FOREACH(goxel->image->layers, layer) {
@@ -298,6 +322,9 @@ void save_to_file(goxel_t *goxel, const char *path)
             chunk_write_dict_value(&c, out, "img-path", layer->image->path,
                                strlen(layer->image->path));
         }
+        if (!box_is_null(layer->box))
+            chunk_write_dict_value(&c, out, "box", &layer->box,
+                                   sizeof(layer->box));
 
         chunk_write_finish(&c, out);
     }
@@ -327,6 +354,43 @@ void save_to_file(goxel_t *goxel, const char *path)
     gzclose(out);
 }
 
+// Iter info of a gox file, without actually reading it.
+// For the moment only returns the image preview if available.
+int gox_iter_infos(const char *path,
+                   int (*callback)(const char *attr, int size,
+                                   void *value, void *user),
+                   void *user)
+{
+    gzFile in;
+    chunk_t c;
+    uint8_t *png;
+    char magic[4];
+
+    in = gzopen(path, "rb");
+
+    gzread(in, magic, 4);
+    assert(strncmp(magic, "GOX ", 4) == 0);
+    read_int32(in);
+
+    while (chunk_read_start(&c, in)) {
+        if (strncmp(c.type, "BL16", 4) == 0) break;
+        if (strncmp(c.type, "LAYR", 4) == 0) break;
+        if (strncmp(c.type, "PREV", 4) == 0) {
+            png = calloc(1, c.length);
+            chunk_read(&c, in, (char*)png, c.length);
+            callback(c.type, c.length, png, user);
+            free(png);
+        } else {
+            // Ignore other blocks.
+            chunk_read(&c, in, NULL, c.length);
+        }
+        chunk_read_finish(&c, in);
+    }
+    gzclose(in);
+    return 0;
+}
+
+
 static block_hash_t *hash_find_at(block_hash_t *hash, int index)
 {
     int i;
@@ -356,6 +420,7 @@ int load_from_file(goxel_t *goxel, const char *path)
 
     LOG_I("Load from file %s", path);
     in = gzopen(path, "rb");
+    if (!in) return -1;
 
     gzread(in, magic, 4);
     assert(strncmp(magic, "GOX ", 4) == 0);
@@ -389,12 +454,7 @@ int load_from_file(goxel_t *goxel, const char *path)
             free(png);
 
         } else if (strncmp(c.type, "LAYR", 4) == 0) {
-            layer = calloc(1, sizeof(*layer));
-            layer->mesh = mesh_new();
-            layer->visible = true;
-            DL_APPEND(goxel->image->layers, layer);
-            goxel->image->active_layer = layer;
-
+            layer = image_add_layer(goxel->image);
             nb_blocks = chunk_read_int32(&c, in);   assert(nb_blocks >= 0);
             for (i = 0; i < nb_blocks; i++) {
                 index = chunk_read_int32(&c, in);   assert(index >= 0);
@@ -420,10 +480,15 @@ int load_from_file(goxel_t *goxel, const char *path)
                 if (strcmp(dict_key, "img-path") == 0) {
                     layer->image = texture_new_image(dict_value, TF_NEAREST);
                 }
-                if (strcmp(dict_key, "id") == 0)
-                    memcpy(&layer->id, dict_value, dict_value_size);
+                if (strcmp(dict_key, "id") == 0) {
+                    typeof(layer->id) id;
+                    memcpy(&id, dict_value, dict_value_size);
+                    if (id) layer->id = id;
+                }
                 if (strcmp(dict_key, "base_id") == 0)
                     memcpy(&layer->base_id, dict_value, dict_value_size);
+                if (strcmp(dict_key, "box") == 0)
+                    memcpy(&layer->box, dict_value, dict_value_size);
             }
         } else if (strncmp(c.type, "CAMR", 4) == 0) {
             camera = camera_new("unnamed");
@@ -450,7 +515,10 @@ int load_from_file(goxel_t *goxel, const char *path)
                 if (strcmp(dict_key, "box") == 0)
                     memcpy(&goxel->image->box, dict_value, dict_value_size);
             }
-        } else assert(false);
+        } else {
+            // Ignore other blocks.
+            chunk_read(&c, in, NULL, c.length);
+        }
         chunk_read_finish(&c, in);
     }
 
@@ -487,7 +555,7 @@ ACTION_REGISTER(open,
     .shortcut = "Ctrl O",
 )
 
-static void save_as(const char *path)
+static void save_as(const char *path, bool with_preview)
 {
     if (!path) {
         path = noc_file_dialog_open(NOC_FILE_DIALOG_SAVE, "gox\0*.gox\0",
@@ -498,23 +566,23 @@ static void save_as(const char *path)
         free(goxel->image->path);
         goxel->image->path = strdup(path);
     }
-    save_to_file(goxel, goxel->image->path);
+    save_to_file(goxel, goxel->image->path, with_preview);
 }
 
 ACTION_REGISTER(save_as,
     .help = "Save the image as",
     .cfunc = save_as,
-    .csig = "vp",
+    .csig = "vpi",
 )
 
-static void save(const char *path)
+static void save(const char *path, bool with_preview)
 {
-    save_as(path ?: goxel->image->path);
+    save_as(path ?: goxel->image->path, with_preview);
 }
 
 ACTION_REGISTER(save,
     .help = "Save the image",
     .cfunc = save,
-    .csig = "vp",
+    .csig = "vpi",
     .shortcut = "Ctrl S"
 )
