@@ -16,6 +16,8 @@
  * goxel.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+// File: goxel.h
+
 #ifndef GOXEL_H
 #define GOXEL_H
 
@@ -41,7 +43,8 @@
 #include <string.h>
 #include <stdarg.h>
 
-#define GOXEL_VERSION_STR "0.7.2"
+#define GOXEL_VERSION_STR "0.7.3"
+#define GOXEL_DEFAULT_THEME "original"
 
 // #### Set the DEBUG macro ####
 #ifndef DEBUG
@@ -159,10 +162,13 @@ enum {
 
 
 
-// ### Some useful inline functions / macros.
+
+// ####### Section: Utils ################################################
+
+// Some useful inline functions / macros
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
-#define SWAP(x0, x) {typeof(x0) tmp = x0; x0 = x; x = tmp;}
+#define SWAP(x0, x) do {typeof(x0) tmp = x0; x0 = x; x = tmp;} while (0)
 
 // Used to pass and get values to callback 'void *user' arguments.
 #define USER_PASS(...) ((const void*[]){__VA_ARGS__})
@@ -179,6 +185,9 @@ enum {
     })
 
 // Some useful vector macros.
+#define vec3_set(v, x, y, z) do { \
+    (v)[0] = (x); (v)[1] = (y); (v)[2] = (z); } while(0)
+
 #define vec4_set(v, x, y, z, w) do { \
     (v)[0] = (x); (v)[1] = (y); (v)[2] = (z); (v)[3] = (w); } while(0)
 
@@ -228,6 +237,16 @@ enum {
     (_a > _b) ? +1 : (_a < _b) ? -1 : 0; \
 })
 
+/* Function: smoothstep
+ * Perform Hermite interpolation between two values.
+ *
+ * This is similar to the smoothstep function in OpenGL shader language.
+ *
+ * Parameters:
+ *   edge0 - Lower edge of the Hermite function.
+ *   edge1 - Upper edge of the Hermite function.
+ *   x     - Source value for interpolation.
+ */
 static inline float smoothstep(float edge0, float edge1, float x)
 {
     x = clamp((x - edge0) / (edge1 - edge0), 0.0f, 1.0f);
@@ -239,6 +258,14 @@ static inline float mix(float x, float y, float t)
     return (1.0 - t) * x + t * y;
 }
 
+/* Function: set_flag
+ * Set some int bits to 0 or 1.
+ *
+ * Parameters:
+ *   x    - The int value to change.
+ *   flag - Bitmask of the bits we want to set.
+ *   v    - Value to set.
+ */
 static inline void set_flag(int *x, int flag, bool v)
 {
     v ? (*x |= flag) : (*x &= ~flag);
@@ -265,6 +292,7 @@ int check_gl_errors(const char *file, int line);
 enum {
     GOX_GL_QCOM_tiled_rendering,
     GOX_GL_OES_packed_depth_stencil,
+    GOX_GL_OES_depth_texture,
     GOX_GL_EXT_discard_framebuffer,
 
     GOX_GL_EXTENSIONS_COUNT
@@ -301,8 +329,22 @@ int list_dir(const char *url, int flags, void *user,
 
 // Convert from screen coordinate to world coordinates.
 // Similar to gluUnproject.
-vec3_t unproject(const vec3_t *win, const mat4_t *model,
-                 const mat4_t *proj, const vec4_t *view);
+void unproject(const float win[3], const float model[4][4],
+               const float proj[4][4], const float viewport[4],
+               float out[3]);
+
+/* Function: b64_decode
+ * Decode a base64 string
+ *
+ * Parameters:
+ *   src  - A base 64 encoded string.
+ *   dest - Buffer that will receive the decoded value or NULL.  If set to
+ *          NULL the function just returns the size of the decoded data.
+ *
+ * Return:
+ *   The size of the decoded data.
+ */
+int b64_decode(const char *src, void *dest);
 
 // #############################
 
@@ -332,7 +374,7 @@ enum {
     DIALOG_FLAG_DIR     = 1 << 2,
 };
 
-// #### Texture ################
+// #### Section: Texture ################
 enum {
     TF_DEPTH    = 1 << 0,
     TF_STENCIL  = 1 << 1,
@@ -345,6 +387,8 @@ enum {
     TF_NEAREST  = 1 << 8,
 };
 
+// Type: texture_t
+// Reresent a 2d texture.
 typedef struct texture texture_t;
 struct texture {
     texture_t   *next;      // All the textures are in a global list.
@@ -361,6 +405,8 @@ struct texture {
 };
 
 texture_t *texture_new_image(const char *path, int flags);
+texture_t *texture_new_from_buf(const uint8_t *data,
+                                int w, int h, int bpp, int flags);
 texture_t *texture_new_surface(int w, int h, int flags);
 texture_t *texture_new_buffer(int w, int h, int flags);
 void texture_get_data(const texture_t *tex, int w, int h, int bpp,
@@ -485,16 +531,6 @@ enum {
 
 
 // #### Tool/Operation/Painter #
-enum {
-    MODE_NULL,
-    MODE_OVER,
-    MODE_SUB,
-    MODE_SUB_CLAMP,
-    MODE_PAINT,
-    MODE_MAX,
-    MODE_INTERSECT,
-    MODE_MULT_ALPHA,
-};
 
 enum {
     TOOL_NONE = 0,
@@ -520,7 +556,7 @@ enum {
 
 typedef struct shape {
     const char *id;
-    float (*func)(const vec3_t *p, const vec3_t *s, float smoothness);
+    float (*func)(const float p[3], const float s[3], float smoothness);
 } shape_t;
 
 void shapes_init(void);
@@ -529,17 +565,6 @@ extern shape_t shape_cube;
 extern shape_t shape_cylinder;
 
 
-// The painting context, including the tool, brush, mode, radius,
-// color, etc...
-typedef struct painter {
-    int             mode;
-    const shape_t   *shape;
-    uint8_t         color[4];
-    float           smoothness;
-    int             symmetry; // bitfield X Y Z
-    box_t           *box;     // Clipping box (can be null)
-} painter_t;
-
 // #### Block ##################
 // The block size can only be 16.
 #define BLOCK_SIZE 16
@@ -561,17 +586,98 @@ typedef struct voxel_vertex
     uint8_t  bump_uv[2]                 __attribute__((aligned(4)));
 } voxel_vertex_t;
 
-// #### Mesh util functions ############################
 
-box_t mesh_get_box(const mesh_t *mesh, bool exact);
-void mesh_op(mesh_t *mesh, painter_t *painter, const box_t *box);
+// ######## Section: Mesh util ############################################
+
+/*
+ * Enum: MODE
+ * Define how layers/brush are merged.  Each mode defines how to apply a
+ * source voxel into a destination voxel.
+ *
+ * MODE_OVER        - New values replace old one.
+ * MODE_SUB         - Substract source alpha from destination
+ * MODE_SUB_CLAMP   - Set alpha to the minimum between the destination value
+ *                    and one minus the source value.
+ * MODE_PAINT       - Set the color of the destination using the source.
+ * MODE_MAX         - Set alpha to the max of the source and destination.
+ * MODE_INTERSECT   - Set alpha to the min of the source and destination.
+ * MODE_MULT_ALPHA  - Multiply the source and dest using source alpha.
+ */
+enum {
+    MODE_NULL,
+    MODE_OVER,
+    MODE_SUB,
+    MODE_SUB_CLAMP,
+    MODE_PAINT,
+    MODE_MAX,
+    MODE_INTERSECT,
+    MODE_MULT_ALPHA,
+};
+
+// Type: painter_t
+// The painting context, including the tool, brush, mode, radius,
+// color, etc...
+//
+// Attributes:
+//   mode - Define how colors are applied.  One of the <MODE> enum value.
+typedef struct painter {
+    int             mode;
+    const shape_t   *shape;
+    uint8_t         color[4];
+    float           smoothness;
+    int             symmetry; // bitfield X Y Z
+    float           (*box)[4][4];     // Clipping box (can be null)
+} painter_t;
+
+
+/* Function: mesh_get_box
+ * Compute the bounding box of a mesh.  */
+// XXX: remove this function!
+void mesh_get_box(const mesh_t *mesh, bool exact, float box[4][4]);
+
+/* Function: mesh_op
+ * Apply a paint operation to a mesh.
+ * This function render geometrical 3d shapes into a mesh.
+ * The shape, mode and color are defined in the painter argument.
+ *
+ * Parameters:
+ *   mesh    - The mesh we paint into.
+ *   painter - Defines the paint operation to apply.
+ *   box     - Defines the position and size of the shape as the
+ *             transformation matrix from the zero centered unit box.
+ *
+ * See Also:
+ *   <painter_t>
+ */
+void mesh_op(mesh_t *mesh, const painter_t *painter, const float box[4][4]);
+
 // XXX: to cleanup.
-void mesh_extrude(mesh_t *mesh, const plane_t *plane, const box_t *box);
+void mesh_extrude(mesh_t *mesh,
+                  const float plane[4][4],
+                  const float box[4][4]);
 
+/* Function: mesh_blit
+ *
+ * Blit voxel data into a mesh.
+ * This is the fastest way to quickly put data into a mesh.
+ *
+ * Parameters:
+ *   mesh - The mesh we blit into.
+ *   data - Pointer to voxel data (RGBA values, in xyz order).
+ *   x    - X pos.
+ *   y    - Y pos.
+ *   z    - Z pos.
+ *   w    - Width of the data.
+ *   h    - Height of the data.
+ *   d    - Depth of the data.
+ *   iter - Optional iterator for optimized access.
+ */
 void mesh_blit(mesh_t *mesh, const uint8_t *data,
                int x, int y, int z, int w, int h, int d,
                mesh_iterator_t *iter);
-void mesh_move(mesh_t *mesh, const mat4_t *mat);
+
+void mesh_move(mesh_t *mesh, const float mat[4][4]);
+
 void mesh_shift_alpha(mesh_t *mesh, int v);
 
 // Compute the selection mask for a given condition.
@@ -583,11 +689,23 @@ int mesh_select(const mesh_t *mesh,
                             void *user),
                 void *user, mesh_t *selection);
 
-void mesh_merge(mesh_t *mesh, const mesh_t *other, int op);
+void mesh_merge(mesh_t *mesh, const mesh_t *other, int op,
+                const uint8_t color[4]);
 
 int mesh_generate_vertices(const mesh_t *mesh, const int block_pos[3],
                            int effects, voxel_vertex_t *out);
 
+// XXX: use int[2][3] for the box?
+void mesh_crop(mesh_t *mesh, const float box[4][4]);
+
+/* Function: mesh_crc32
+ * Compute the crc32 of the mesh data as an array of xyz rgba values.
+ *
+ * This is only used in the tests, to make sure that we can still open
+ * old file formats.
+ */
+uint32_t mesh_crc32(const mesh_t *mesh);
+
 // #### Renderer ###############
 
 enum {
@@ -622,8 +740,8 @@ typedef struct renderer renderer_t;
 typedef struct render_item_t render_item_t;
 struct renderer
 {
-    mat4_t view_mat;
-    mat4_t proj_mat;
+    float view_mat[4][4];
+    float proj_mat[4][4];
     int    fbo;     // The renderer target framebuffer.
     float  scale;   // For retina display.
 
@@ -642,16 +760,16 @@ struct renderer
 void render_init(void);
 void render_deinit(void);
 void render_mesh(renderer_t *rend, const mesh_t *mesh, int effects);
-void render_plane(renderer_t *rend, const plane_t *plane,
+void render_plane(renderer_t *rend, const float plane[4][4],
                   const uint8_t color[4]);
-void render_line(renderer_t *rend, const vec3_t *a, const vec3_t *b,
+void render_line(renderer_t *rend, const float a[3], const float b[3],
                  const uint8_t color[4]);
-void render_box(renderer_t *rend, const box_t *box,
+void render_box(renderer_t *rend, const float box[4][4],
                 const uint8_t color[4], int effects);
-void render_sphere(renderer_t *rend, const mat4_t *mat);
-void render_img(renderer_t *rend, texture_t *tex, const mat4_t *mat,
+void render_sphere(renderer_t *rend, const float mat[4][4]);
+void render_img(renderer_t *rend, texture_t *tex, const float mat[4][4],
                 int efffects);
-void render_rect(renderer_t *rend, const plane_t *plane, int effects);
+void render_rect(renderer_t *rend, const float plane[4][4], int effects);
 // Flushes all the queued render items.  Actually calls opengl.
 //  rect: the viewport rect (passed to glViewport).
 //  clear_color: clear the screen with this first.
@@ -659,7 +777,7 @@ void render_submit(renderer_t *rend, const int rect[4],
                    const uint8_t clear_color[4]);
 int render_get_default_settings(int i, char **name, render_settings_t *out);
 // Compute the light direction in the model coordinates (toward the light)
-vec3_t render_get_light_dir(const renderer_t *rend);
+void render_get_light_dir(const renderer_t *rend, float out[3]);
 
 // Ugly function that return the position of the block at a given id
 // when the mesh is rendered with render_mesh.
@@ -674,10 +792,10 @@ void render_get_block_pos(renderer_t *rend, const mesh_t *mesh,
 // #### Model3d ################
 
 typedef struct {
-     vec3_t   pos       __attribute__((aligned(4)));
-     vec3_t   normal    __attribute__((aligned(4)));
+     float    pos[3]    __attribute__((aligned(4)));
+     float    normal[3] __attribute__((aligned(4)));
      uint8_t  color[4]  __attribute__((aligned(4)));
-     vec2_t   uv        __attribute__((aligned(4)));
+     float    uv[2]     __attribute__((aligned(4)));
 } model_vertex_t;
 
 typedef struct {
@@ -702,10 +820,10 @@ model3d_t *model3d_line(void);
 model3d_t *model3d_rect(void);
 model3d_t *model3d_wire_rect(void);
 void model3d_render(model3d_t *model3d,
-                    const mat4_t *model, const mat4_t *proj,
+                    const float model[4][4], const float proj[4][4],
                     const uint8_t color[4],
                     const texture_t *tex,
-                    const vec3_t *light,
+                    const float light[3],
                     int   effects);
 
 // #### Palette ################
@@ -773,7 +891,7 @@ enum {
 // `down` represent each button in the mouse.  For touch events only the
 // first element is set.
 typedef struct {
-    vec2_t  pos;
+    float   pos[2];
     bool    down[3];
 } touch_t;
 
@@ -815,16 +933,17 @@ struct gesture
     int     type;
     int     button;
     int     state;
-    vec4_t  view;
-    vec2_t  pos;
-    vec2_t  start_pos[2];
+    float   viewport[4];
+    float   pos[2];
+    float   start_pos[2][2];
     float   pinch;
     float   rotation;
     int     (*callback)(const gesture_t *gest, void *user);
 };
 
 int gesture_update(int nb, gesture_t *gestures[],
-                   const inputs_t *inputs, const vec4_t *view, void *user);
+                   const inputs_t *inputs, const float viewport[4],
+                   void *user);
 
 
 // #############################
@@ -836,14 +955,14 @@ struct camera
     char   name[128];  // 127 chars max.
     bool   ortho; // Set to true for orthographic projection.
     float  dist;
-    quat_t rot;
-    vec3_t ofs;
+    float  rot[4]; // Quaternion.
+    float  ofs[3];
     float  fovy;
     float  aspect;
 
     // Auto computed from other values:
-    mat4_t view_mat;    // Model to view transformation.
-    mat4_t proj_mat;    // Proj transform from camera coordinates.
+    float view_mat[4][4];    // Model to view transformation.
+    float proj_mat[4][4];    // Proj transform from camera coordinates.
 };
 
 camera_t *camera_new(const char *name);
@@ -852,15 +971,18 @@ void camera_set(camera_t *camera, const camera_t *other);
 void camera_update(camera_t *camera);
 // Adjust the camera settings so that the rotation works for a given
 // position.
-void camera_set_target(camera_t *camera, const vec3_t *pos);
+void camera_set_target(camera_t *camera, const float pos[3]);
 
+// Function: camera_get_ray
 // Get the raytracing ray of the camera at a given screen position.
-// win:     pixel position in screen coordinates.
-// view:    viewport rect: [min_x, min_y, max_x, max_y].
-// o:       output ray origin.
-// d:       output ray direction.
-void camera_get_ray(const camera_t *camera, const vec2_t *win,
-                    const vec4_t *view, vec3_t *o, vec3_t *d);
+//
+// Parameters:
+//   win   - Pixel position in screen coordinates.
+//   view  - Viewport rect: [min_x, min_y, max_x, max_y].
+//   o     - Output ray origin.
+//   d     - Output ray direction.
+void camera_get_ray(const camera_t *camera, const float win[2],
+                    const float viewport[4], float o[3], float d[3]);
 
 typedef struct history history_t;
 
@@ -870,13 +992,14 @@ struct layer {
     mesh_t      *mesh;
     int         id;         // Uniq id in the image (for clones).
     bool        visible;
-    char        name[128];  // 127 chars max.
-    mat4_t      mat;
+    char        name[256];  // 256 chars max.
+    float       box[4][4];  // Bounding box.
+    float       mat[4][4];
     // For 2d image layers.
     texture_t   *image;
     // For clone layers:
     int         base_id;
-    uint64_t    base_mesh_id;
+    uint64_t    base_mesh_key;
 };
 
 typedef struct image image_t;
@@ -885,7 +1008,7 @@ struct image {
     layer_t *active_layer;
     camera_t *cameras;
     camera_t *active_camera;
-    box_t    box;
+    float    box[4][4];
 
     // For saving.
     char    *path;
@@ -933,7 +1056,7 @@ typedef struct proc {
 
 int proc_parse(const char *txt, gox_proc_t *proc);
 void proc_release(gox_proc_t *proc);
-int proc_start(gox_proc_t *proc, const box_t *box);
+int proc_start(gox_proc_t *proc, const float box[4][4]);
 int proc_stop(gox_proc_t *proc);
 int proc_iter(gox_proc_t *proc);
 
@@ -953,9 +1076,11 @@ enum {
     CURSOR_CTRL         = 1 << 2,
 };
 
+typedef struct gesture3d gesture3d_t;
+
 typedef struct cursor {
-    vec3_t pos;
-    vec3_t normal;
+    float  pos[3];
+    float  normal[3];
     int    snap_mask;
     int    snaped;
     int    flags; // Union of CURSOR_* values.
@@ -963,11 +1088,11 @@ typedef struct cursor {
 } cursor_t;
 
 // #### 3d gestures
-typedef struct gesture3d gesture3d_t;
 struct gesture3d
 {
     int         type;
     int         state;
+    int         buttons; // CURSOR_SHIFT | CURSOR_CTRL
     cursor_t    *cursor;
     int         (*callback)(gesture3d_t *gest, void *user);
 };
@@ -978,6 +1103,7 @@ enum {
     // Tools flags.
     TOOL_REQUIRE_CAN_EDIT = 1 << 0, // Set to tools that can edit the layer.
     TOOL_REQUIRE_CAN_MOVE = 1 << 1, // Set to tools that can move the layer.
+    TOOL_ALLOW_PICK_COLOR = 1 << 2, // Ctrl switches to pick color tool.
 };
 
 // Tools
@@ -985,7 +1111,7 @@ typedef struct tool tool_t;
 struct tool {
     int id;
     const char *action_id;
-    int (*iter_fn)(tool_t *tool, const vec4_t *view);
+    int (*iter_fn)(tool_t *tool, const float viewport[4]);
     int (*gui_fn)(tool_t *tool);
     const char *shortcut;
     int state; // XXX: to be removed I guess.
@@ -1004,8 +1130,7 @@ struct tool {
     }
 
 void tool_register_(const tool_t *tool);
-tool_t *tool_get(int id);
-int tool_iter(tool_t *tool, const vec4_t *view);
+int tool_iter(tool_t *tool, const float viewport[4]);
 int tool_gui(tool_t *tool);
 
 int tool_gui_snap(void);
@@ -1025,20 +1150,21 @@ typedef struct goxel
     image_t    *image;
 
     mesh_t     *layers_mesh; // All the layers combined.
-    mesh_t     *pick_mesh;   // Used for picking (always layers_mesh?)
-    mesh_t     *tool_mesh;   // Preview of the tool action.
+    // Tools can set this mesh and it will replace the current layer mesh
+    // during render.
+    mesh_t     *tool_mesh;
     mesh_t     *render_mesh; // All the layers + tool mesh.
 
     struct     {
         mesh_t *mesh;
-        box_t  box;
+        float  box[4][4];
     } clipboard;
 
     history_t  *history;     // Undo/redo history.
     int        snap_mask;    // Global snap mask (can edit in the GUI).
     float      snap_offset;  // Only for brush tool, remove that?
 
-    plane_t    plane;         // The snapping plane.
+    float      plane[4][4];         // The snapping plane.
     bool       plane_hidden;  // Set to true to hide the plane.
     bool       show_export_viewport;
 
@@ -1058,15 +1184,15 @@ typedef struct goxel
     float      tool_radius;
 
     // Some state for the tool iter functions.
-    plane_t    tool_plane;
+    float      tool_plane[4][4];
     bool       tool_shape_two_steps; // Param of the shape tool.
 
-    box_t      selection;   // The selection box.
+    float      selection[4][4];   // The selection box.
 
     struct {
-        quat_t rotation;
-        vec2_t pos;
-        vec3_t camera_ofs;
+        float  rotation[4];
+        float  pos[2];
+        float  camera_ofs[3];
     } move_origin;
 
     gox_proc_t proc;        // The current procedural rendering (if any).
@@ -1076,8 +1202,9 @@ typedef struct goxel
     char       *help_text;  // Seen in the bottom of the screen.
     char       *hint_text;  // Seen in the bottom of the screen.
 
-    int        frame_count;       // Global frames counter.
-    double     frame_time;        // Clock time at beginning of the frame.
+    int        frame_count; // Global frames counter.
+    double     frame_time;  // Clock time at beginning of the frame (sec)
+    double     fps;         // Average fps.
     bool       quit;        // Set to true to quit the application.
 
     struct {
@@ -1096,27 +1223,27 @@ void goxel_init(goxel_t *goxel);
 void goxel_release(goxel_t *goxel);
 void goxel_iter(goxel_t *goxel, inputs_t *inputs);
 void goxel_render(goxel_t *goxel);
-void goxel_render_view(goxel_t *goxel, const vec4_t *rect);
+void goxel_render_view(goxel_t *goxel, const float viewport[4]);
 // Called by the gui when the mouse hover a 3D view.
 // XXX: change the name since we also call it when the mouse get out of
 // the view.
-void goxel_mouse_in_view(goxel_t *goxel, const vec4_t *view,
+void goxel_mouse_in_view(goxel_t *goxel, const float viewport[4],
                          const inputs_t *inputs);
 
-int goxel_unproject(goxel_t *goxel, const vec4_t *view,
-                    const vec2_t *pos, int snap_mask, float offset,
-                    vec3_t *out, vec3_t *normal);
+int goxel_unproject(goxel_t *goxel, const float viewport[4],
+                    const float pos[2], int snap_mask, float offset,
+                    float out[3], float normal[3]);
 
-bool goxel_unproject_on_mesh(goxel_t *goxel, const vec4_t *view,
-                     const vec2_t *pos, mesh_t *mesh,
-                     vec3_t *out, vec3_t *normal);
+bool goxel_unproject_on_mesh(goxel_t *goxel, const float viewport[4],
+                     const float pos[2], mesh_t *mesh,
+                     float out[3], float normal[3]);
 
-bool goxel_unproject_on_plane(goxel_t *goxel, const vec4_t *view,
-                     const vec2_t *pos, const plane_t *plane,
-                     vec3_t *out, vec3_t *normal);
-bool goxel_unproject_on_box(goxel_t *goxel, const vec4_t *view,
-                     const vec2_t *pos, const box_t *box, bool inside,
-                     vec3_t *out, vec3_t *normal, int *face);
+bool goxel_unproject_on_plane(goxel_t *goxel, const float viewport[4],
+                     const float pos[2], const float plane[4][4],
+                     float out[3], float normal[3]);
+bool goxel_unproject_on_box(goxel_t *goxel, const float viewport[4],
+                     const float pos[2], const float box[4][4], bool inside,
+                     float out[3], float normal[3], int *face);
 // Recompute the meshes.  mask from MESH_ enum.
 void goxel_update_meshes(goxel_t *goxel, int mask);
 
@@ -1125,14 +1252,21 @@ void goxel_set_hint_text(goxel_t *goxel, const char *msg, ...);
 
 void goxel_import_image_plane(goxel_t *goxel, const char *path);
 
-// Render the view into an RGBA buffer.
-void goxel_render_to_buf(uint8_t *buf, int w, int h);
+// Render the view into an RGB[A] buffer.
+void goxel_render_to_buf(uint8_t *buf, int w, int h, int bpp);
 
 // #############################
 
-void save_to_file(goxel_t *goxel, const char *path);
+void save_to_file(goxel_t *goxel, const char *path, bool with_preview);
 int load_from_file(goxel_t *goxel, const char *path);
 
+// Iter info of a gox file, without actually reading it.
+// For the moment only returns the image preview if available.
+int gox_iter_infos(const char *path,
+                   int (*callback)(const char *attr, int size,
+                                   void *value, void *user),
+                   void *user);
+
 
 // #### Colors functions #######
 void hsl_to_rgb(const uint8_t hsl[3], uint8_t rgb[3]);
@@ -1216,7 +1350,7 @@ void gui_iter(goxel_t *goxel, const inputs_t *inputs);
 void gui_render(void);
 
 // Gui widgets:
-void gui_text(const char *label);
+void gui_text(const char *label, ...);
 bool gui_button(const char *label, float w, int icon);
 void gui_group_begin(const char *label);
 void gui_group_end(void);
@@ -1225,7 +1359,8 @@ bool gui_input_int(const char *label, int *v, int minv, int maxv);
 bool gui_input_float(const char *label, float *v, float step,
                      float minv, float maxv, const char *format);
 bool gui_angle(const char *id, float *v, int vmin, int vmax);
-bool gui_quat(const char *label, quat_t *q);
+bool gui_bbox(float box[4][4]);
+bool gui_quat(const char *label, float q[4]);
 bool gui_action_button(const char *id, const char *label, float size,
                        const char *sig, ...);
 bool gui_action_checkbox(const char *id, const char *label);
@@ -1248,7 +1383,21 @@ enum {
     GUI_POPUP_RESIZE    = 1 << 1,
 };
 
-void gui_open_popup(const char *title, bool (*func)(void), int flags);
+
+/*
+ * Function: gui_open_popup
+ * Open a modal popup.
+ *
+ * Parameters:
+ *    title - The title of the popup.
+ *    flags - Union of <GUI_POPUP_FLAGS> values.
+ *    data  - Data passed to the popup.  It will be automatically released
+ *            by the gui.
+ *    func  - The popup function, that render the popup gui.  Should return
+ *            true to close the popup.
+ */
+void gui_open_popup(const char *title, int flags, void *data,
+                    bool (*func)(void *data));
 void gui_popup_body_begin(void);
 void gui_popup_body_end(void);
 
@@ -1283,10 +1432,21 @@ void mustache_free(mustache_t *m);
 // ####### Cache manager #########################
 // Allow to cache blocks merge operations.
 typedef struct cache cache_t;
-
+// Create a new cache with a given max size (in byte).
 cache_t *cache_create(int size);
+// Add an item into the cache.
+// Inputs:
+//  key, keylen     Define the unique key for the cache item.
+//  data            Pointer to the item data.  The cache takes ownership.
+//  cost            Cost of the data used to compute the cache usage.
+//                  It doesn't have to be the size.
+//  delfunc         Function that the cache can use to free the data.
 void cache_add(cache_t *cache, const void *key, int keylen, void *data,
                int cost, int (*delfunc)(void *data));
+// Return an item from the cache.
+// Returns
+//  The data owned by the cache, or NULL if no item with this key is in
+//  the cache.
 void *cache_get(cache_t *cache, const void *key, int keylen);
 
 // ####### Sound #################################
@@ -1294,5 +1454,11 @@ void sound_init(void);
 void sound_play(const char *sound);
 void sound_iter(void);
 
+// Section: tests
+
+/* Function: tests_run
+ * Run all the unit tests */
+void tests_run(void);
+
 
 #endif // GOXEL_H
