@@ -33,7 +33,7 @@ extern "C" {
 #include "imgui_internal.h"
 
 extern "C" {
-    bool gui_settings_popup(void);
+    bool gui_settings_popup(void *data);
 }
 
 static inline ImVec4 color_lighten(ImVec4 c, float k)
@@ -94,7 +94,7 @@ static const char *FSHADER =
 ;
 
 typedef struct {
-    vec4_t  rect;
+    float  rect[4];
 } view_t;
 
 typedef struct {
@@ -126,8 +126,9 @@ typedef struct gui_t {
 
     struct {
         const char *title;
-        bool      (*func)(void);
+        bool      (*func)(void *data);
         int         flags;
+        void       *data;
     } popup;
 
     // Textures for the color editor.
@@ -470,7 +471,7 @@ static int on_gesture(const gesture_t *gest, void *user)
 {
     gui_t *gui = (gui_t*)user;
     ImGuiIO& io = ImGui::GetIO();
-    io.MousePos = ImVec2(gest->pos.x, gest->pos.y);
+    io.MousePos = ImVec2(gest->pos[0], gest->pos[1]);
     io.MouseDown[0] = (gest->type == GESTURE_DRAG) &&
                       (gest->state != GESTURE_END);
     if (gest->state == GESTURE_BEGIN && !gui->mouse_in_view)
@@ -503,26 +504,28 @@ void gui_release(void)
 }
 
 // XXX: Move this somewhere else.
-void render_axis_arrows(goxel_t *goxel, const vec2_t *view_size)
+void render_axis_arrows(goxel_t *goxel, const float view_size[2])
 {
-    const vec3_t AXIS[] = {vec3(1, 0, 0), vec3(0, 1, 0), vec3(0, 0, 1)};
+    const float AXIS[][3] = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
     int i;
     const int d = 40;  // Distance to corner of the view.
-    vec2_t spos = vec2(d, d);
-    vec3_t pos, normal, b;
+    float spos[2] = {d, d};
+    float pos[3], normal[3], b[3];
     uint8_t c[4];
     float s = 1;
-    vec4_t view = vec4(0, 0, view_size->x, view_size->y);
-    camera_get_ray(&goxel->camera, &spos, &view, &pos, &normal);
+    float view[4] = {0, 0, view_size[0], view_size[1]};
+    camera_get_ray(&goxel->camera, spos, view, pos, normal);
     if (goxel->camera.ortho)
         s = goxel->camera.dist / 32;
     else
-        vec3_iaddk(&pos, normal, 100);
+        vec3_iaddk(pos, normal, 100);
 
     for (i = 0; i < 3; i++) {
-        b = vec3_addk(pos, AXIS[i], 2.0 * s);
-        vec4_set(c, AXIS[i].x * 255, AXIS[i].y * 255, AXIS[i].z * 255, 255);
-        render_line(&goxel->rend, &pos, &b, c);
+        vec3_addk(pos, AXIS[i], 2.0 * s, b);
+        vec4_set(c, AXIS[i][0] * 255,
+                    AXIS[i][1] * 255,
+                    AXIS[i][2] * 255, 255);
+        render_line(&goxel->rend, pos, b, c);
     }
 }
 
@@ -532,11 +535,11 @@ void render_view(const ImDrawList* parent_list, const ImDrawCmd* cmd)
     view_t *view = (view_t*)cmd->UserCallbackData;
     const float width = ImGui::GetIO().DisplaySize.x;
     const float height = ImGui::GetIO().DisplaySize.y;
-    int rect[4] = {(int)view->rect.x,
-                   (int)(height - view->rect.y - view->rect.w),
-                   (int)view->rect.z,
-                   (int)view->rect.w};
-    goxel_render_view(goxel, &view->rect);
+    int rect[4] = {(int)view->rect[0],
+                   (int)(height - view->rect[1] - view->rect[3]),
+                   (int)view->rect[2],
+                   (int)view->rect[3]};
+    goxel_render_view(goxel, view->rect);
     render_submit(&goxel->rend, rect, goxel->back_color);
     GL(glViewport(0, 0, width * scale, height * scale));
 }
@@ -685,8 +688,8 @@ static bool layer_item(int i, int icon, bool *visible, bool *edit,
 static void layers_panel(goxel_t *goxel)
 {
     layer_t *layer;
-    int i = 0, icon;
-    bool current, visible;
+    int i = 0, icon, bbox[2][3];
+    bool current, visible, bounded;
     gui_group_begin(NULL);
     DL_FOREACH(goxel->image->layers, layer) {
         current = goxel->image->active_layer == layer;
@@ -716,18 +719,35 @@ static void layers_panel(goxel_t *goxel)
     ImGui::SameLine();
     gui_action_button("img_move_layer_down", NULL, 0, "");
 
+    layer = goxel->image->active_layer;
+    bounded = !box_is_null(layer->box);
+
     gui_group_begin(NULL);
     gui_action_button("img_duplicate_layer", "Duplicate", 1, "");
     gui_action_button("img_clone_layer", "Clone", 1, "");
     gui_action_button("img_merge_visible_layers", "Merge visible", 1, "");
+    if (bounded && gui_button("Crop to box", 1, 0)) {
+        mesh_crop(layer->mesh, layer->box);
+        goxel_update_meshes(goxel, -1);
+    }
     gui_group_end();
 
-    if (goxel->image->active_layer->base_id) {
+    if (layer->base_id) {
         gui_group_begin(NULL);
         gui_action_button("img_unclone_layer", "Unclone", 1, "");
         gui_action_button("img_select_parent_layer", "Select parent", 1, "");
         gui_group_end();
     }
+    if (ImGui::Checkbox("Bounded", &bounded)) {
+        if (bounded) {
+            mesh_get_bbox(layer->mesh, bbox, true);
+            if (bbox[0][0] > bbox[1][0]) memset(bbox, 0, sizeof(bbox));
+            bbox_from_aabb(layer->box, bbox);
+        } else {
+            mat4_copy(mat4_zero, layer->box);
+        }
+    }
+    if (bounded) gui_bbox(layer->box);
 }
 
 
@@ -914,38 +934,17 @@ static void export_panel(goxel_t *goxel)
 static void image_panel(goxel_t *goxel)
 {
     bool bounded;
-    int x, y, z, w, h, d;
     image_t *image = goxel->image;
-    box_t *box = &image->box;
+    float (*box)[4][4] = &image->box;
 
     bounded = !box_is_null(*box);
     if (ImGui::Checkbox("Bounded", &bounded)) {
-        *box = bounded ? bbox_from_extents(vec3_zero, 16, 16, 16) : box_null;
-    }
-    if (bounded) {
-        w = box->w.x * 2;
-        h = box->h.y * 2;
-        d = box->d.z * 2;
-        x = round(box->p.x - box->w.x);
-        y = round(box->p.y - box->h.y);
-        z = round(box->p.z - box->d.z);
-
-        gui_group_begin("Origin");
-        gui_input_int("x", &x, 0, 0);
-        gui_input_int("y", &y, 0, 0);
-        gui_input_int("z", &z, 0, 0);
-        gui_group_end();
-
-        gui_group_begin("Size");
-        gui_input_int("w", &w, 1, 2048);
-        gui_input_int("h", &h, 1, 2048);
-        gui_input_int("d", &d, 1, 2048);
-        gui_group_end();
-
-        *box = bbox_from_extents(
-                vec3(x + w / 2., y + h / 2., z + d / 2.),
-                w / 2., h / 2., d / 2.);
+        if (bounded)
+            bbox_from_extents(*box, vec3_zero, 16, 16, 16);
+        else
+            mat4_copy(mat4_zero, *box);
     }
+    if (bounded) gui_bbox(*box);
 }
 
 static void cameras_panel(goxel_t *goxel)
@@ -979,12 +978,12 @@ static void cameras_panel(goxel_t *goxel)
     gui_input_float("dist", &cam->dist, 10.0, 0, 0, NULL);
 
     gui_group_begin("Offset");
-    gui_input_float("x", &cam->ofs.x, 1.0, 0, 0, NULL);
-    gui_input_float("y", &cam->ofs.y, 1.0, 0, 0, NULL);
-    gui_input_float("z", &cam->ofs.z, 1.0, 0, 0, NULL);
+    gui_input_float("x", &cam->ofs[0], 1.0, 0, 0, NULL);
+    gui_input_float("y", &cam->ofs[1], 1.0, 0, 0, NULL);
+    gui_input_float("z", &cam->ofs[2], 1.0, 0, 0, NULL);
     gui_group_end();
 
-    gui_quat("Rotation", &cam->rot);
+    gui_quat("Rotation", cam->rot);
 
     gui_group_begin("Set");
     gui_action_button("view_left", "left", 0.5, ""); ImGui::SameLine();
@@ -995,6 +994,11 @@ static void cameras_panel(goxel_t *goxel)
     gui_group_end();
 }
 
+static void debug_panel(goxel_t *goxel)
+{
+    ImGui::Text("FPS: %d", (int)round(goxel->fps));
+}
+
 static void import_image_plane(goxel_t *goxel)
 {
     const char *path;
@@ -1004,7 +1008,7 @@ static void import_image_plane(goxel_t *goxel)
     goxel_import_image_plane(goxel, path);
 }
 
-static bool shift_alpha_popup(void)
+static bool shift_alpha_popup(void *data)
 {
     static int v = 0;
     static mesh_t *original_mesh = NULL;
@@ -1025,7 +1029,7 @@ static bool shift_alpha_popup(void)
     return false;
 }
 
-static bool about_popup(void)
+static bool about_popup(void *data)
 {
     using namespace ImGui;
     Text("Goxel " GOXEL_VERSION_STR);
@@ -1135,10 +1139,10 @@ static void render_menu(void)
         render_menu_item("copy", "Copy");
         render_menu_item("past", "Past");
         if (ImGui::MenuItem("Shift Alpha"))
-            gui_open_popup("Shift Alpha", shift_alpha_popup, 0);
+            gui_open_popup("Shift Alpha", 0, NULL, shift_alpha_popup);
         if (ImGui::MenuItem("Settings"))
-            gui_open_popup("Settings", gui_settings_popup,
-                    GUI_POPUP_FULL | GUI_POPUP_RESIZE);
+            gui_open_popup("Settings", GUI_POPUP_FULL | GUI_POPUP_RESIZE,
+                           NULL, gui_settings_popup);
         ImGui::EndMenu();
     }
     if (ImGui::BeginMenu("View")) {
@@ -1151,7 +1155,7 @@ static void render_menu(void)
     }
     if (ImGui::BeginMenu("Help")) {
         if (ImGui::MenuItem("About"))
-            gui_open_popup("About", about_popup, 0);
+            gui_open_popup("About", 0, NULL, about_popup);
         ImGui::EndMenu();
     }
     ImGui::EndMenuBar();
@@ -1220,6 +1224,7 @@ static void render_left_panel(void)
         {"Cameras", cameras_panel},
         {"Image", image_panel},
         {"Export", export_panel},
+        {"Debug", debug_panel},
     };
     ImDrawList* draw_list;
 
@@ -1268,8 +1273,8 @@ void gui_iter(goxel_t *goxel, const inputs_t *inputs)
     ImGuiStyle& style = ImGui::GetStyle();
     const theme_t *theme = theme_get();
     gesture_t *gestures[] = {&gui->gestures.drag, &gui->gestures.hover};
-    vec4_t display_rect = vec4(0, 0,
-                               goxel->screen_size[0], goxel->screen_size[1]);
+    float display_rect[4] = {
+        0.f, 0.f, (float)goxel->screen_size[0], (float)goxel->screen_size[1]};
     float font_size = ImGui::GetFontSize();
 
     io.DisplaySize = ImVec2((float)goxel->screen_size[0],
@@ -1278,7 +1283,7 @@ void gui_iter(goxel_t *goxel, const inputs_t *inputs)
     io.DisplayFramebufferScale = ImVec2(goxel->screen_scale,
                                         goxel->screen_scale);
     io.DeltaTime = 1.0 / 60;
-    gesture_update(2, gestures, inputs, &display_rect, gui);
+    gesture_update(2, gestures, inputs, display_rect, gui);
     io.MouseWheel = inputs->mouse_wheel;
 
     for (i = 0; i < ARRAY_SIZE(inputs->keys); i++)
@@ -1348,12 +1353,14 @@ void gui_iter(goxel_t *goxel, const inputs_t *inputs)
         }
         if (ImGui::BeginPopupModal(gui->popup.title, NULL, flags)) {
             typeof(gui->popup.func) func = gui->popup.func;
-            if (func()) {
+            if (func(gui->popup.data)) {
                 ImGui::CloseCurrentPopup();
                 // Only reset if there was no change.
                 if (func == gui->popup.func) {
                     gui->popup.title = NULL;
                     gui->popup.func = NULL;
+                    free(gui->popup.data);
+                    gui->popup.data = NULL;
                 }
             }
             ImGui::EndPopup();
@@ -1366,29 +1373,29 @@ void gui_iter(goxel_t *goxel, const inputs_t *inputs)
     ImVec2 canvas_pos = ImGui::GetCursorScreenPos();
     ImVec2 canvas_size = ImGui::GetContentRegionAvail();
     canvas_size.y -= 20; // Leave space for the help label.
-    gui->view.rect = vec4(canvas_pos.x, canvas_pos.y,
-                          canvas_size.x, canvas_size.y);
+    vec4_set(gui->view.rect, canvas_pos.x, canvas_pos.y,
+                             canvas_size.x, canvas_size.y);
     draw_list = ImGui::GetWindowDrawList();
     draw_list->AddCallback(render_view, &gui->view);
     // Invisible button so that we catch inputs.
     ImGui::InvisibleButton("canvas", canvas_size);
     gui->mouse_in_view = ImGui::IsItemHovered();
-    vec2_t view_size = vec2(gui->view.rect.z, gui->view.rect.w);
-    vec4_t view_rect = vec4(canvas_pos.x,
-                            io.DisplaySize.y - (canvas_pos.y + canvas_size.y),
-                            canvas_size.x, canvas_size.y);
+    float view_size[2] = {gui->view.rect[2], gui->view.rect[3]};
+    float view_rect[4] = {canvas_pos.x,
+                          io.DisplaySize.y - (canvas_pos.y + canvas_size.y),
+                          canvas_size.x, canvas_size.y};
     // Call mouse_in_view with inputs in the view referential.
     if (!(!gui->mouse_in_view && inputs->mouse_wheel) &&
             !gui->capture_mouse) {
         inputs_t inputs2 = *inputs;
         for (i = 0; i < ARRAY_SIZE(inputs->touches); i++) {
-            inputs2.touches[i].pos.y =
-                io.DisplaySize.y - inputs2.touches[i].pos.y;
+            inputs2.touches[i].pos[1] =
+                io.DisplaySize.y - inputs2.touches[i].pos[1];
         }
-        goxel_mouse_in_view(goxel, &view_rect, &inputs2);
+        goxel_mouse_in_view(goxel, view_rect, &inputs2);
     }
 
-    render_axis_arrows(goxel, &view_size);
+    render_axis_arrows(goxel, view_size);
     ImGui::Text("%s", goxel->hint_text ?: "");
     ImGui::SameLine(180);
     ImGui::Text("%s", goxel->help_text ?: "");
@@ -1478,6 +1485,7 @@ void gui_group_begin(const char *label)
 void gui_group_end(void)
 {
     gui->group--;
+    ImGui::PopID();
     ImGui::PopStyleVar(2);
     ImGui::Dummy(ImVec2(0, 0));
     ImGui::EndGroup();
@@ -1497,7 +1505,6 @@ void gui_group_end(void)
     // Stencil reset.
     draw_list->AddCallback(stencil_callback, (void*)(intptr_t)0);
     GoxBox2(pos, size, COLOR(WIDGET, OUTLINE, 0), false);
-    ImGui::PopID();
 }
 
 bool gui_input_int(const char *label, int *v, int minv, int maxv)
@@ -1537,6 +1544,36 @@ bool gui_input_float(const char *label, float *v, float step,
     return ret;
 }
 
+bool gui_bbox(float box[4][4])
+{
+    int x, y, z, w, h, d;
+    bool ret = false;
+    float p[3];
+    w = box[0][0] * 2;
+    h = box[1][1] * 2;
+    d = box[2][2] * 2;
+    x = round(box[3][0] - box[0][0]);
+    y = round(box[3][1] - box[1][1]);
+    z = round(box[3][2] - box[2][2]);
+
+    gui_group_begin("Origin");
+    ret |= gui_input_int("x", &x, 0, 0);
+    ret |= gui_input_int("y", &y, 0, 0);
+    ret |= gui_input_int("z", &z, 0, 0);
+    gui_group_end();
+    gui_group_begin("Size");
+    ret |= gui_input_int("w", &w, 1, 2048);
+    ret |= gui_input_int("h", &h, 1, 2048);
+    ret |= gui_input_int("d", &d, 1, 2048);
+    gui_group_end();
+
+    if (ret) {
+        vec3_set(p, x + w / 2., y + h / 2., z + d / 2.);
+        bbox_from_extents(box, p, w / 2., h / 2., d / 2.);
+    }
+    return ret;
+}
+
 bool gui_angle(const char *id, float *v, int vmin, int vmax)
 {
     int a;
@@ -1669,9 +1706,12 @@ float gui_get_avail_width(void)
     return GetContentRegionAvailWidth();
 }
 
-void gui_text(const char *label)
+void gui_text(const char *label, ...)
 {
-    Text("%s", label);
+    va_list args;
+    va_start(args, label);
+    TextV(label, args);
+    va_end(args);
 }
 
 void gui_same_line(void)
@@ -1793,42 +1833,44 @@ void gui_enabled_end(void)
     ImGui::PopStyleColor();
 }
 
-bool gui_quat(const char *label, quat_t *q)
+bool gui_quat(const char *label, float q[4])
 {
     // Hack to prevent weird behavior when we change the euler angles.
     // We keep track of the last used euler angles value and reuse them if
     // the quaternion is the same.
     static struct {
-        quat_t quat;
-        vec3_t eul;
+        float quat[4];
+        float eul[3];
     } last = {};
-
-    vec3_t eul;
+    float eul[3];
     bool ret = false;
 
-    if (memcmp(q, &last.quat, sizeof(*q)) == 0)
-        eul = last.eul;
+    if (memcmp(q, &last.quat, sizeof(last.quat)) == 0)
+        vec3_copy(last.eul, eul);
     else
-        eul = quat_to_eul(*q, EULER_ORDER_DEFAULT);
+        quat_to_eul(q, EULER_ORDER_DEFAULT, eul);
     gui_group_begin(label);
-    if (gui_angle("x", &eul.x, -180, +180)) ret = true;
-    if (gui_angle("y", &eul.y, -180, +180)) ret = true;
-    if (gui_angle("z", &eul.z, -180, +180)) ret = true;
+    if (gui_angle("x", &eul[0], -180, +180)) ret = true;
+    if (gui_angle("y", &eul[1], -180, +180)) ret = true;
+    if (gui_angle("z", &eul[2], -180, +180)) ret = true;
     gui_group_end();
 
     if (ret) {
-        *q = eul_to_quat(eul, EULER_ORDER_DEFAULT);
-        last.quat = *q;
-        last.eul = eul;
+        eul_to_quat(eul, EULER_ORDER_DEFAULT, q);
+        quat_copy(q, last.quat);
+        vec3_copy(eul, last.eul);
     }
     return ret;
 }
 
-void gui_open_popup(const char *title, bool (*func)(void), int flags)
+void gui_open_popup(const char *title, int flags, void *data,
+                    bool (*func)(void *data))
 {
     gui->popup.title = title;
     gui->popup.func = func;
     gui->popup.flags = flags;
+    assert(!gui->popup.data);
+    gui->popup.data = data;
 }
 
 void gui_popup_body_begin(void) {
