@@ -77,7 +77,7 @@ static layer_t *layer_new(const image_t *img, const char *name)
     // XXX: potential bug here.
     strncpy(layer->name, name, sizeof(layer->name));
     layer->mesh = mesh_new();
-    layer->mat = mat4_identity;
+    mat4_set_identity(layer->mat);
     layer->id = img_get_new_id(img);
     return layer;
 }
@@ -90,10 +90,11 @@ static layer_t *layer_copy(layer_t *other)
     layer->visible = other->visible;
     layer->mesh = mesh_copy(other->mesh);
     layer->image = texture_copy(other->image);
-    layer->mat = other->mat;
+    mat4_copy(other->box, layer->box);
+    mat4_copy(other->mat, layer->mat);
     layer->id = other->id;
     layer->base_id = other->base_id;
-    layer->base_mesh_id = other->base_mesh_id;
+    layer->base_mesh_key = other->base_mesh_key;
     return layer;
 }
 
@@ -106,9 +107,9 @@ static layer_t *layer_clone(layer_t *other)
     snprintf(layer->name, sizeof(layer->name), "%.*s clone", len, other->name);
     layer->visible = other->visible;
     layer->mesh = mesh_copy(other->mesh);
-    layer->mat = mat4_identity;
+    mat4_set_identity(layer->mat);
     layer->base_id = other->id;
-    layer->base_mesh_id = mesh_get_id(other->mesh);
+    layer->base_mesh_key = mesh_get_key(other->mesh);
     return layer;
 }
 
@@ -118,10 +119,10 @@ void image_update(image_t *img)
     layer_t *layer, *base;
     DL_FOREACH(img->layers, layer) {
         base = img_get_layer(img, layer->base_id);
-        if (base && layer->base_mesh_id != mesh_get_id(base->mesh)) {
+        if (base && layer->base_mesh_key != mesh_get_key(base->mesh)) {
             mesh_set(layer->mesh, base->mesh);
-            mesh_move(layer->mesh, &layer->mat);
-            layer->base_mesh_id = mesh_get_id(base->mesh);
+            mesh_move(layer->mesh, layer->mat);
+            layer->base_mesh_key = mesh_get_key(base->mesh);
         }
     }
 }
@@ -139,7 +140,6 @@ image_t *image_new(void)
     image_t *img = calloc(1, sizeof(*img));
     img->export_width = 1024;
     img->export_height = 1024;
-    img->box = box_null;
     layer = layer_new(img, "background");
     layer->visible = true;
     DL_APPEND(img->layers, layer);
@@ -297,7 +297,7 @@ void image_merge_visible_layers(image_t *img)
     DL_FOREACH(img->layers, layer) {
         if (!layer->visible) continue;
         if (last) {
-            mesh_merge(layer->mesh, last->mesh, MODE_OVER);
+            mesh_merge(layer->mesh, last->mesh, MODE_OVER, NULL);
             DL_DELETE(img->layers, last);
             layer_delete(last);
         }
@@ -407,10 +407,15 @@ void image_history_push(image_t *img)
     debug_print_history(img);
 }
 
+// XXX: not clear what this is doing.  We should try to remove it.
+// It swap the content of two images without touching their pointer or
+// history.
 static void swap(image_t *a, image_t *b)
 {
-    SWAP(a->layers, b->layers);
-    SWAP(a->active_layer, b->active_layer);
+    SWAP(*a, *b);
+    SWAP(a->history, b->history);
+    SWAP(a->history_next, b->history_next);
+    SWAP(a->history_prev, b->history_prev);
 }
 
 void image_undo(image_t *img)
@@ -441,11 +446,11 @@ void image_redo(image_t *img)
     debug_print_history(img);
 }
 
-void image_clear_layer(layer_t *layer, const box_t *box)
+void image_clear_layer(layer_t *layer, const float box[4][4])
 {
     painter_t painter;
     layer = layer ?: goxel->image->active_layer;
-    if (!box || box_is_null(*box)) {
+    if (!box || box_is_null(box)) {
         mesh_clear(layer->mesh);
         return;
     }
