@@ -24,12 +24,6 @@
 #   define DEBUG_ONLY(x)
 #endif
 
-// For the moment I put an implementation using GLFW3 and one using GLUT.
-// Both have some annoying problems, so I am not sure which one I should
-// keep.  By default I use GLFW3, which is the that works the best so far.
-
-#ifndef USE_GLUT
-
 #ifdef GLES2
 #   define GLFW_INCLUDE_ES2
 #endif
@@ -38,6 +32,7 @@
 static goxel_t      *g_goxel = NULL;
 static inputs_t     *g_inputs = NULL;
 static GLFWwindow   *g_window = NULL;
+static float        g_scale = 1;
 
 void on_scroll(GLFWwindow *win, double x, double y)
 {
@@ -61,6 +56,7 @@ typedef struct
 {
     char *input;
     char *export;
+    float scale;
 } args_t;
 
 #ifndef NO_ARGP
@@ -72,6 +68,7 @@ static char doc[] = "A 3D voxels editor";
 static char args_doc[] = "[INPUT]";
 static struct argp_option options[] = {
     {"export",   'e', "FILENAME", 0, "Export the model to a file" },
+    {"scale",    's', "FLOAT", 0, "Set UI scale (for retina display)"},
     {},
 };
 
@@ -85,6 +82,9 @@ static error_t parse_opt(int key, char *arg, struct argp_state *state)
     case 'e':
         args->export = arg;
         break;
+    case 's':
+        args->scale = atof(arg);
+        break;
     case ARGP_KEY_ARG:
         if (state->arg_num >= 1)
             argp_usage(state);
@@ -107,6 +107,7 @@ static void loop_function(void) {
     int fb_size[2], win_size[2];
     int i;
     double xpos, ypos;
+    float scale = g_scale;
 
     if (    !glfwGetWindowAttrib(g_window, GLFW_VISIBLE) ||
              glfwGetWindowAttrib(g_window, GLFW_ICONIFIED)) {
@@ -118,9 +119,9 @@ static void loop_function(void) {
     // size.
     glfwGetWindowSize(g_window, &win_size[0], &win_size[1]);
     glfwGetFramebufferSize(g_window, &fb_size[0], &fb_size[1]);
-    g_inputs->window_size[0] = win_size[0];
-    g_inputs->window_size[1] = win_size[1];
-    g_inputs->scale = fb_size[0] / win_size[0];
+    g_inputs->window_size[0] = win_size[0] / scale;
+    g_inputs->window_size[1] = win_size[1] / scale;
+    g_inputs->scale = fb_size[0] / win_size[0] * scale;
 
     GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
 
@@ -128,7 +129,7 @@ static void loop_function(void) {
         g_inputs->keys[i] = glfwGetKey(g_window, i) == GLFW_PRESS;
     }
     glfwGetCursorPos(g_window, &xpos, &ypos);
-    g_inputs->touches[0].pos = vec2(xpos, ypos);
+    vec2_set(g_inputs->touches[0].pos, xpos / scale, ypos / scale);
     g_inputs->touches[0].down[0] =
         glfwGetMouseButton(g_window, GLFW_MOUSE_BUTTON_LEFT) == GLFW_PRESS;
     g_inputs->touches[0].down[1] =
@@ -191,7 +192,7 @@ static void set_window_icon(GLFWwindow *window) {}
 
 int main(int argc, char **argv)
 {
-    args_t args = {};
+    args_t args = {.scale = 1};
     GLFWwindow *window;
     GLFWmonitor *monitor;
     const GLFWvidmode *mode;
@@ -204,6 +205,7 @@ int main(int argc, char **argv)
 #ifndef NO_ARGP
     argp_parse (&argp, argc, argv, 0, 0, &args);
 #endif
+    g_scale = args.scale;
 
     glfwInit();
     glfwWindowHint(GLFW_SAMPLES, 2);
@@ -221,6 +223,8 @@ int main(int argc, char **argv)
     glewInit();
 #endif
     goxel_init(g_goxel);
+    // Run the unit tests in debug.
+    if (DEBUG) tests_run();
 
     if (args.input)
         action_exec2("import", "p", args.input);
@@ -238,138 +242,3 @@ end:
     goxel_release(g_goxel);
     return ret;
 }
-
-#else // GLUT implementation
-
-#ifdef __APPLE__
-#   include <GLUT/glut.h>
-#else
-#   include <GL/glut.h>
-#endif
-
-
-static goxel_t  *g_goxel = NULL;
-static inputs_t *g_inputs = NULL;
-
-static const int KEYS[] = {
-    [GLUT_KEY_RIGHT]        = KEY_RIGHT,
-    [GLUT_KEY_LEFT]         = KEY_LEFT,
-    [GLUT_KEY_DOWN]         = KEY_DOWN,
-    [GLUT_KEY_UP]           = KEY_UP,
-};
-
-static void on_display(void)
-{
-}
-
-static void on_reshape(GLint width, GLint height)
-{
-    g_inputs->window_size[0] = width;
-    g_inputs->window_size[1] = height;
-}
-
-static void set_modifiers(void)
-{
-    int v = glutGetModifiers();
-    g_inputs->keys[KEY_SHIFT] = v & GLUT_ACTIVE_SHIFT;
-    g_inputs->keys[KEY_CONTROL] = v & GLUT_ACTIVE_CTRL;
-}
-
-static void on_key_down(unsigned char key, int x, int y)
-{
-    int i;
-    for (i = 0; i < ARRAY_SIZE(g_inputs->chars); i++) {
-        if (!g_inputs->chars[i]) {
-            g_inputs->chars[i] = key;
-            break;
-        }
-    }
-    set_modifiers();
-}
-
-static void on_key_up(unsigned char key, int x, int y)
-{
-    set_modifiers();
-}
-
-static void on_special_key_down(int key, int x, int y)
-{
-    if (key < ARRAY_SIZE(KEYS))
-        g_inputs->keys[KEYS[key]] = true;
-}
-
-static void on_special_key_up(int key, int x, int y)
-{
-    if (key < ARRAY_SIZE(KEYS))
-        g_inputs->keys[KEYS[key]] = false;
-}
-
-static void on_mouse_button(int button, int state, int x, int y)
-{
-    if (button < 3) {
-        g_inputs->mouse_pos = vec2(x, y);
-        g_inputs->mouse_down[button] = state == GLUT_DOWN;
-    }
-    if (button == 3 && state == GLUT_DOWN)
-        g_inputs->mouse_wheel = +1;
-    if (button == 4 && state == GLUT_DOWN)
-        g_inputs->mouse_wheel = -1;
-    set_modifiers();
-}
-
-static void on_mouse_motion(int x, int y)
-{
-    g_inputs->mouse_pos = vec2(x, y);
-}
-
-static void on_passive_motion(int x, int y)
-{
-    g_inputs->mouse_pos = vec2(x, y);
-}
-
-static void on_timer(int value)
-{
-    GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
-    goxel_iter(g_goxel, g_inputs);
-    g_inputs->mouse_wheel = 0;
-    memset(g_inputs->chars, 0, sizeof(g_inputs->chars));
-    goxel_render(g_goxel);
-    glutSwapBuffers();
-    glutTimerFunc(10, on_timer, 0);
-}
-
-int main(int argc, char **argv)
-{
-    int w = 640;
-    int h = 480;
-    goxel_t goxel;
-    inputs_t inputs;
-
-    g_goxel = &goxel;
-    g_inputs = &inputs;
-    memset(g_inputs, 0, sizeof(*g_inputs));
-    g_inputs->scale = 1;
-
-    glutInit(&argc, argv);
-    glutInitWindowSize(w, h);
-    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
-    glutCreateWindow("Goxel " GOXEL_VERSION_STR);
-
-    goxel_init(&goxel);
-
-    glutDisplayFunc(on_display);
-    glutReshapeFunc(on_reshape);
-    glutKeyboardFunc(on_key_down);
-    glutKeyboardUpFunc(on_key_up);
-    glutSpecialFunc(on_special_key_down);
-    glutSpecialUpFunc(on_special_key_up);
-    glutMouseFunc(on_mouse_button);
-    glutMotionFunc(on_mouse_motion);
-    glutPassiveMotionFunc(on_passive_motion);
-    glutTimerFunc(10, on_timer, 0);
-
-    glutMainLoop();
-    return 0;
-}
-
-#endif
