@@ -61,7 +61,7 @@ struct mesh
 {
     block_t *blocks;
     int *ref;   // Used to implement copy on write of the blocks.
-    uint64_t id;     // global uniq id, change each time a mesh changes.
+    uint64_t key; // Two meshes with the same key have the same value.
 };
 
 static uint64_t g_uid = 2; // Global id counter.
@@ -223,41 +223,66 @@ static void block_get_at(const block_t *block, const int pos[3],
     memcpy(out, BLOCK_AT(block, x, y, z), 4);
 }
 
-static void block_set_at(block_t *block, const int pos[3], const uint8_t v[4])
-{
-    int x, y, z;
-    block_prepare_write(block);
-    x = pos[0] - block->pos[0];
-    y = pos[1] - block->pos[1];
-    z = pos[2] - block->pos[2];
-    assert(x >= 0 && x < N);
-    assert(y >= 0 && y < N);
-    assert(z >= 0 && z < N);
-    memcpy(BLOCK_AT(block, x, y, z), v, 4);
-}
-
-static void mesh_get_bbox(const mesh_t *mesh, int bbox[2][3])
+/*
+ * Function: mesh_get_bbox
+ *
+ * Get the bounding box of a mesh.
+ *
+ * Inputs:
+ *   mesh   - The mesh
+ *   exact  - If true, compute the exact bounding box.  If false, returns
+ *            an approximation that might be slightly bigger than the
+ *            actual box, but faster to compute.
+ *
+ * Outputs:
+ *   bbox  - The bounding box as the bottom left and top right corner of
+ *           the mesh.  If the mesh is empty, this will contain all zero.
+ *
+ * Returns:
+ *   true if the mesh is not empty.
+ */
+bool mesh_get_bbox(const mesh_t *mesh, int bbox[2][3], bool exact)
 {
     block_t *block;
     int ret[2][3] = {{INT_MAX, INT_MAX, INT_MAX},
                      {INT_MIN, INT_MIN, INT_MIN}};
-    for (block = mesh->blocks; block; block = block->hh.next) {
-        if (block_is_empty(block, true)) continue;
-        ret[0][0] = min(ret[0][0], block->pos[0]);
-        ret[0][1] = min(ret[0][1], block->pos[1]);
-        ret[0][2] = min(ret[0][2], block->pos[2]);
-        ret[1][0] = max(ret[1][0], block->pos[0] + N);
-        ret[1][1] = max(ret[1][1], block->pos[1] + N);
-        ret[1][2] = max(ret[1][2], block->pos[1] + N);
+    int pos[3];
+    mesh_iterator_t iter;
+    bool empty = false;
+
+    if (!exact) {
+        for (block = mesh->blocks; block; block = block->hh.next) {
+            if (block_is_empty(block, true)) continue;
+            ret[0][0] = min(ret[0][0], block->pos[0]);
+            ret[0][1] = min(ret[0][1], block->pos[1]);
+            ret[0][2] = min(ret[0][2], block->pos[2]);
+            ret[1][0] = max(ret[1][0], block->pos[0] + N);
+            ret[1][1] = max(ret[1][1], block->pos[1] + N);
+            ret[1][2] = max(ret[1][2], block->pos[1] + N);
+        }
+    } else {
+        iter = mesh_get_iterator(mesh, MESH_ITER_SKIP_EMPTY);
+        while (mesh_iter(&iter, pos)) {
+            if (!mesh_get_alpha_at(mesh, &iter, pos)) continue;
+            ret[0][0] = min(ret[0][0], pos[0]);
+            ret[0][1] = min(ret[0][1], pos[1]);
+            ret[0][2] = min(ret[0][2], pos[2]);
+            ret[1][0] = max(ret[1][0], pos[0] + 1);
+            ret[1][1] = max(ret[1][1], pos[1] + 1);
+            ret[1][2] = max(ret[1][2], pos[2] + 1);
+        }
     }
+    empty = ret[0][0] >= ret[1][0];
+    if (empty) memset(ret, 0, sizeof(ret));
     memcpy(bbox, ret, sizeof(ret));
+    return !empty;
 }
 
 static void mesh_prepare_write(mesh_t *mesh)
 {
     block_t *blocks, *block, *new_block;
     assert(*mesh->ref > 0);
-    mesh->id = g_uid++;
+    mesh->key = g_uid++;
     if (*mesh->ref == 1)
         return;
     (*mesh->ref)--;
@@ -319,7 +344,7 @@ mesh_t *mesh_new(void)
     mesh_t *mesh;
     mesh = calloc(1, sizeof(*mesh));
     mesh->ref = calloc(1, sizeof(*mesh->ref));
-    mesh->id = g_uid++;
+    mesh->key = 1; // Empty mesh key.
     *mesh->ref = 1;
     return mesh;
 }
@@ -354,7 +379,7 @@ mesh_iterator_t mesh_get_box_iterator(const mesh_t *mesh,
     box_get_bbox(iter.box, iter.bbox);
 
     if (flags & MESH_ITER_SKIP_EMPTY) {
-        mesh_get_bbox(mesh, mesh_bbox);
+        mesh_get_bbox(mesh, mesh_bbox, false);
         bbox_intersection(mesh_bbox, iter.bbox, iter.bbox);
     }
 
@@ -376,6 +401,7 @@ void mesh_clear(mesh_t *mesh)
         HASH_DEL(mesh->blocks, block);
         block_delete(block);
     }
+    mesh->key = 1; // Empty mesh key.
     mesh->blocks = NULL;
 }
 
@@ -399,7 +425,7 @@ mesh_t *mesh_copy(const mesh_t *other)
     mesh_t *mesh = calloc(1, sizeof(*mesh));
     mesh->blocks = other->blocks;
     mesh->ref = other->ref;
-    mesh->id = other->id;
+    mesh->key = other->key;
     (*mesh->ref)++;
     return mesh;
 }
@@ -419,6 +445,7 @@ void mesh_set(mesh_t *mesh, const mesh_t *other)
     }
     mesh->blocks = other->blocks;
     mesh->ref = other->ref;
+    mesh->key = other->key;
     (*mesh->ref)++;
 }
 
@@ -434,7 +461,6 @@ static block_t *mesh_get_block_at(const mesh_t *mesh, const int pos[3],
     int p[3] = {pos[0] & ~(int)(N - 1),
                 pos[1] & ~(int)(N - 1),
                 pos[2] & ~(int)(N - 1)};
-
     if (!it) {
         HASH_FIND(hh, mesh->blocks, p, 3 * sizeof(int), block);
         return block;
@@ -464,10 +490,28 @@ static block_t *mesh_add_block(mesh_t *mesh, const int pos[3])
     return block;
 }
 
-void mesh_get_at(const mesh_t *mesh, mesh_iterator_t *iter,
+void mesh_get_at(const mesh_t *mesh, mesh_iterator_t *it,
                  const int pos[3], uint8_t out[4])
 {
-    block_t *block = mesh_get_block_at(mesh, pos, iter);
+    block_t *block;
+    int p[3];
+
+    if (it && it->block_id && it->block_id == get_block_id(it->block)) {
+        p[0] = pos[0] - it->block_pos[0];
+        p[1] = pos[1] - it->block_pos[1];
+        p[2] = pos[2] - it->block_pos[2];
+        if (    p[0] >= 0 && p[0] < N &&
+                p[1] >= 0 && p[1] < N &&
+                p[2] >= 0 && p[2] < N) {
+            if (!it->block)
+                memset(out, 0, 4);
+            else
+                memcpy(out, BLOCK_AT(it->block, p[0], p[1], p[2]), 4);
+            return;
+        }
+    }
+
+    block = mesh_get_block_at(mesh, pos, it);
     return block_get_at(block, pos, out);
 }
 
@@ -489,7 +533,15 @@ void mesh_set_at(mesh_t *mesh, mesh_iterator_t *iter,
             vec3_copy(p, iter->block_pos);
         }
     }
-    return block_set_at(block, pos, v);
+
+    block_prepare_write(block);
+    p[0] = pos[0] - block->pos[0];
+    p[1] = pos[1] - block->pos[1];
+    p[2] = pos[2] - block->pos[2];
+    assert(p[0] >= 0 && p[0] < N);
+    assert(p[1] >= 0 && p[1] < N);
+    assert(p[2] >= 0 && p[2] < N);
+    memcpy(BLOCK_AT(block, p[0], p[1], p[2]), v, 4);
 }
 
 
@@ -506,7 +558,7 @@ static bool mesh_iter_next_block_box(mesh_iterator_t *it)
 
     for (i = 0; i < 3; i++) {
         it->block_pos[i] += N;
-        if (it->block_pos[i] < it->bbox[1][i]) break;
+        if (it->block_pos[i] <= it->bbox[1][i]) break;
         it->block_pos[i] = it->bbox[0][i] & ~(int)(N - 1);
     }
     if (i == 3) return false;
@@ -562,6 +614,8 @@ int mesh_iter(mesh_iterator_t *it, int pos[3])
 {
     int i;
     if (!it->block_id) { // First call.
+        // XXX: this is not good: mesh_iter shouldn't make change to the
+        // mesh.
         if (it->flags & MESH_ITER_INCLUDES_NEIGHBORS)
             mesh_add_neighbors_blocks((mesh_t*)it->mesh);
         if (!mesh_iter_next_block(it)) return 0;
@@ -577,18 +631,21 @@ int mesh_iter(mesh_iterator_t *it, int pos[3])
 
 next_block:
     if (!mesh_iter_next_block(it)) {
-        mesh_remove_empty_blocks((mesh_t*)it->mesh, true);
+        // XXX: this is not good: mesh_iter shouldn't make changes to the
+        // mesh.
+        if (it->flags & MESH_ITER_INCLUDES_NEIGHBORS)
+            mesh_remove_empty_blocks((mesh_t*)it->mesh, true);
         return 0;
     }
 
 end:
-    vec3_copy(it->pos, pos);
+    if (pos) vec3_copy(it->pos, pos);
     return 1;
 }
 
-uint64_t mesh_get_id(const mesh_t *mesh)
+uint64_t mesh_get_key(const mesh_t *mesh)
 {
-    return mesh->id;
+    return mesh ? mesh->key : 0;
 }
 
 void *mesh_get_block_data(const mesh_t *mesh, mesh_accessor_t *iter,
@@ -641,7 +698,7 @@ void mesh_read(const mesh_t *mesh,
 
     block_t *block;
     int block_pos[3] = {pos[0] + 1, pos[1] + 1, pos[2] + 1};
-    int z, y, x, dx, dy, dz, p[3];
+    int i, z, y, x, dx, dy, dz, p[3];
     uint8_t v[4];
     mesh_accessor_t accessor;
 
@@ -662,13 +719,48 @@ void mesh_read(const mesh_t *mesh,
 
 rest:
     // Fill the rest.
+    // In order to optimize access, we iter in such a way to avoid changing
+    // the current block: start with the planes, then the edges, and finally
+    // the 8 corners.
     accessor = mesh_get_accessor(mesh);
-    for (z = 0; z < N + 2; z++)
-    for (y = 0; y < N + 2; y++)
-    for (x = 0; x < N + 2; x++) {
-        if ((z > 1 && z < N + 1) &&
-            (y > 1 && y < N + 1) &&
-            (x > 1 && x < N + 1)) continue;
+    const int ranges[26][3][2] = {
+        // 6 planes.
+        {{0,         1}, {1,     N + 1}, {1,     N + 1}},
+        {{N + 1, N + 2}, {1,     N + 1}, {1,     N + 1}},
+        {{1,     N + 1}, {0,         1}, {1,     N + 1}},
+        {{1,     N + 1}, {N + 1, N + 2}, {1,     N + 1}},
+        {{1,     N + 1}, {1,     N + 1}, {0,         1}},
+        {{1,     N + 1}, {1,     N + 1}, {N + 1, N + 2}},
+        // 4 edges moving along X.
+        {{1,     N + 1}, {0,         1}, {0,         1}},
+        {{1,     N + 1}, {N + 1, N + 2}, {0,         1}},
+        {{1,     N + 1}, {0,         1}, {N + 1, N + 2}},
+        {{1,     N + 1}, {N + 1, N + 2}, {N + 1, N + 2}},
+        // 4 edges moving along Y.
+        {{0,         1}, {1,     N + 1}, {0,         1}},
+        {{N + 1, N + 2}, {1,     N + 1}, {0,         1}},
+        {{0,         1}, {1,     N + 1}, {1,     N + 2}},
+        {{N + 1, N + 2}, {1,     N + 1}, {1,     N + 2}},
+        // 4 edges moving along Z.
+        {{0,         1}, {0,         1}, {1,     N + 1}},
+        {{N + 1, N + 2}, {0,         1}, {1,     N + 1}},
+        {{0,         1}, {N + 1, N + 2}, {1,     N + 1}},
+        {{N + 1, N + 2}, {N + 1, N + 2}, {1,     N + 1}},
+        // 8 Corners.
+        {{0,     1    }, {0,     1    }, {0,     1    }},
+        {{N + 1, N + 2}, {0,     1    }, {0,     1    }},
+        {{0,     1    }, {N + 1, N + 2}, {0,     1    }},
+        {{N + 1, N + 2}, {N + 1, N + 2}, {0,     1    }},
+        {{0,     1    }, {0,     1    }, {N + 1, N + 2}},
+        {{N + 1, N + 2}, {0,     1    }, {N + 1, N + 2}},
+        {{0,     1    }, {N + 1, N + 2}, {N + 1, N + 2}},
+        {{N + 1, N + 2}, {N + 1, N + 2}, {N + 1, N + 2}},
+    };
+
+    for (i = 0; i < 26; i++)
+    for (z = ranges[i][2][0]; z < ranges[i][2][1]; z++)
+    for (y = ranges[i][1][0]; y < ranges[i][1][1]; y++)
+    for (x = ranges[i][0][0]; x < ranges[i][0][1]; x++) {
         p[0] = pos[0] + x;
         p[1] = pos[1] + y;
         p[2] = pos[2] + z;
