@@ -21,7 +21,7 @@
 
 typedef struct {
     tool_t tool;
-    box_t box;
+    float  box[4][4];
 
     struct {
         gesture3d_t drag;
@@ -42,7 +42,7 @@ static int on_drag(gesture3d_t *gest, void *user)
     if (gest->state == GESTURE_BEGIN)
         image_history_push(goxel->image);
 
-    mesh_op(mesh, &painter, &laser->box);
+    mesh_op(mesh, &painter, laser->box);
     goxel_update_meshes(goxel, MESH_RENDER);
 
     if (gest->state == GESTURE_END)
@@ -51,12 +51,14 @@ static int on_drag(gesture3d_t *gest, void *user)
     return 0;
 }
 
-static int iter(tool_t *tool, const vec4_t *view)
+static int iter(tool_t *tool, const float viewport[4])
 {
     tool_laser_t *laser = (tool_laser_t*)tool;
     cursor_t *curs = &goxel->cursor;
     curs->snap_mask = SNAP_CAMERA;
     curs->snap_offset = 0;
+    float v[4];
+    float view_mat_inv[4][4];
 
     if (!laser->gestures.drag.type) {
         laser->gestures.drag = (gesture3d_t) {
@@ -66,19 +68,20 @@ static int iter(tool_t *tool, const vec4_t *view)
     }
 
     // Create the tool box from the camera along the visible ray.
-    laser->box.mat = mat4_identity;
-    laser->box.w = mat4_mul_vec(mat4_inverted(goxel->camera.view_mat),
-                                vec4(1, 0, 0, 0)).xyz;
-    laser->box.h = mat4_mul_vec(mat4_inverted(goxel->camera.view_mat),
-                                vec4(0, 1, 0, 0)).xyz;
-    laser->box.d = mat4_mul_vec(mat4_inverted(goxel->camera.view_mat),
-                                vec4(0, 0, 1, 0)).xyz;
-    laser->box.d = vec3_neg(curs->normal);
-    laser->box.p = curs->pos;
+    mat4_set_identity(laser->box);
+    mat4_invert(goxel->camera.view_mat, view_mat_inv);
+    mat4_mul_vec4(view_mat_inv, VEC(1, 0, 0, 0), v);
+    vec3_copy(v, laser->box[0]);
+    mat4_mul_vec4(view_mat_inv, VEC(0, 1, 0, 0), v);
+    vec3_copy(v, laser->box[1]);
+    mat4_mul_vec4(view_mat_inv, VEC(0, 0, 1, 0), v);
+    vec3_copy(v, laser->box[2]);
+    vec3_neg(curs->normal, laser->box[2]);
+    vec3_copy(curs->pos, laser->box[3]);
     // Just a large value for the size of the laser box.
-    mat4_itranslate(&laser->box.mat, 0, 0, -1024);
-    mat4_iscale(&laser->box.mat, goxel->tool_radius, goxel->tool_radius, 1024);
-    render_box(&goxel->rend, &laser->box, NULL, EFFECT_WIREFRAME);
+    mat4_itranslate(laser->box, 0, 0, -1024);
+    mat4_iscale(laser->box, goxel->tool_radius, goxel->tool_radius, 1024);
+    render_box(&goxel->rend, laser->box, NULL, EFFECT_WIREFRAME);
 
     gesture3d(&laser->gestures.drag, curs, laser);
 
